@@ -1,7 +1,7 @@
 # encoding: utf-8
 
 # Nagstamon - Nagios status monitor for your desktop
-# Copyright (C) 2008-2013 Henri Wahl <h.wahl@ifw-dresden.de> et al.
+# Copyright (C) 2008-2014 Henri Wahl <h.wahl@ifw-dresden.de> et al.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,28 @@ import sys
 import ConfigParser
 import base64
 import zlib
-import sys
+
+# global flag for keyring availabilty
+keyring_available = False
+# Determine if keyring module and an implementation is available for secure password storage
+
+try:
+    # Linux systems should use keyring only if it comes with the distro, otherwise chances are small
+    # that keyring works at all
+    if platform.system() == "Linux":
+        # keyring and secretstorage have to be importable
+        import keyring, secretstorage
+        if ("SecretService") in dir(keyring.backends) and not (keyring.get_keyring() is None):
+			keyring_available = True
+    else:
+        # nagstamon.spec always have to match module path
+        import Nagstamon.thirdparty.keyring as keyring
+        keyring_available = not (keyring.get_keyring() is None)
+except:
+    import traceback
+    traceback.print_exc(file=sys.stdout)
+    keyring_available = False
+
 
 class Config(object):
     """
@@ -35,11 +56,12 @@ class Config(object):
             supposed to be sensible defaults
         """
         # move from minute interval to seconds
-        #self.update_interval = 1
         self.update_interval_seconds = 60
         self.short_display = False
         self.long_display = True
         self.show_grid = True
+        self.show_tooltips = True
+        self.highlight_new_events = True
         self.default_sort_field = "Status"
         self.default_sort_order = "Ascending"
         self.filter_all_down_hosts = False
@@ -75,8 +97,7 @@ class Config(object):
         self.notification = True
         self.notification_flashing = True
         self.notification_desktop = False
-        # because of nonexistent windows systray popup support I'll let it be now
-        #self.notification_popup = False
+        self.notification_actions = False
         self.notification_sound = True
         self.notification_sound_repeat = False
         self.notification_default_sound = True
@@ -84,11 +105,29 @@ class Config(object):
         self.notification_custom_sound_warning = None
         self.notification_custom_sound_critical = None
         self.notification_custom_sound_down = None
+        self.notification_action_warning = False
+        self.notification_action_warning_string = ""
+        self.notification_action_critical = False
+        self.notification_action_critical_string = ""
+        self.notification_action_down = False
+        self.notification_action_down_string = ""
+        self.notification_action_ok = False
+        self.notification_action_ok_string = ""
+        self.notification_custom_action = False
+        self.notification_custom_action_string = False
+        self.notification_custom_action_separator = False
+        self.notification_custom_action_single = False
         self.notify_if_warning = True
         self.notify_if_critical = True
         self.notify_if_unknown = True
         self.notify_if_unreachable = True
         self.notify_if_down = True
+        """
+        # not yet working
+        # Check_MK show-only-my-problems-they-are-way-enough feature
+        self.only_my_issues = False
+        """
+        # Regular expression filters
         self.re_host_enabled = False
         self.re_host_pattern = ""
         self.re_host_reverse = False
@@ -98,9 +137,6 @@ class Config(object):
         self.re_status_information_enabled = False
         self.re_status_information_pattern = ""
         self.re_status_information_reverse = False
-        self.re_criticality_enabled = False
-        self.re_criticality_pattern = ""
-        self.re_criticality_reverse = False
         self.color_ok_text = self.default_color_ok_text = "#FFFFFF"
         self.color_ok_background = self.default_color_ok_background = "#006400"
         self.color_warning_text = self.default_color_warning_text = "#000000"
@@ -136,6 +172,12 @@ class Config(object):
         self.defaults_downtime_type_flexible = False
         self.converted_from_single_configfile = False
 
+        # Special FX
+        # Centreon
+        self.re_criticality_enabled = False
+        self.re_criticality_pattern = ""
+        self.re_criticality_reverse = False
+
         # the app is unconfigured by default and will stay so if it
         # would not find a config file
         self.unconfigured = True
@@ -300,17 +342,39 @@ class Config(object):
         # time saving config
         try:
             for server in servers:
+                # usernames for monitor server and proxy
                 servers[server].username = self.DeObfuscate(servers[server].username)
+                servers[server].proxy_username = self.DeObfuscate(servers[server].proxy_username)
+                # passwords for monitor server and proxy
                 if servers[server].save_password == "False":
                     servers[server].password = ""
-                else:
+                elif keyring_available:
+                    password = keyring.get_password("Nagstamon", "@".join((servers[server].username,
+                                                                           servers[server].monitor_url))) or ""
+                    if password == "":
+                        if servers[server].password != "":
+                            servers[server].password = self.DeObfuscate(servers[server].password)
+                    else:
+                        servers[server].password = password
+                elif servers[server].password != "":
                     servers[server].password = self.DeObfuscate(servers[server].password)
+                # proxy password
+                if keyring_available:
+                    proxy_password = keyring.get_password("Nagstamon", "@".join(("proxy",
+                                                                                 servers[server].proxy_username,
+                                                                                 servers[server].proxy_address))) or ""
+                    if proxy_password == "":
+                        if servers[server].proxy_password != "":
+                            servers[server].proxy_password = self.DeObfuscate(servers[server].proxy_password)
+                    else:
+                        servers[server].proxy_password = proxy_password
+                elif servers[server].proxy_password != "":
+                    servers[server].proxy_password = self.DeObfuscate(servers[server].proxy_password)
+
                 # do only deobfuscating if any autologin_key is set - will be only Centreon
                 if servers[server].__dict__.has_key("autologin_key"):
                     if len(servers[server].__dict__["autologin_key"]) > 0:
                         servers[server].autologin_key  = self.DeObfuscate(servers[server].autologin_key)
-                servers[server].proxy_username = self.DeObfuscate(servers[server].proxy_username)
-                servers[server].proxy_password = self.DeObfuscate(servers[server].proxy_password)
         except:
             import traceback
             traceback.print_exc(file=sys.stdout)
@@ -452,9 +516,23 @@ class Config(object):
                     if settingsdir == "servers":
                         if option == "username" or option == "password" or option == "proxy_username" or option == "proxy_password" or option == "autologin_key":
                             value = self.Obfuscate(self.__dict__[settingsdir][s].__dict__[option])
-                            if option == "password" \
-                               and self.__dict__[settingsdir][s].save_password == "False":
-                                value = ""
+                            if option == "password":
+                                if self.__dict__[settingsdir][s].save_password == "False":
+                                    value = ""
+                                elif keyring_available:
+                                    if self.__dict__[settingsdir][s].password != "":
+                                        keyring.set_password("Nagstamon", "@".join((self.__dict__[settingsdir][s].username,
+                                                                                    self.__dict__[settingsdir][s].monitor_url)),
+                                                                                    self.__dict__[settingsdir][s].password)
+                                    value = ""
+                            if option == "proxy_password":
+                                if keyring_available:
+                                    if self.__dict__[settingsdir][s].proxy_password != "":
+                                        keyring.set_password("Nagstamon", "@".join(("proxy",\
+                                                                                    self.__dict__[settingsdir][s].proxy_username,
+                                                                                    self.__dict__[settingsdir][s].proxy_address)),
+                                                                                    self.__dict__[settingsdir][s].proxy_password)
+                                    value = ""
                             config.set(setting + "_" + s, option, value)
                         else:
                             config.set(setting + "_" + s, option, self.__dict__[settingsdir][s].__dict__[option])
@@ -476,7 +554,7 @@ class Config(object):
 
         except:
             import traceback
-            traceback.print_exc(file=sys.stdout)
+            traceback.print_exc(file=stdout)
 
 
     def Convert_Conf_to_Multiple_Servers(self):
@@ -534,13 +612,13 @@ class Config(object):
             self.__dict__.has_key("app_vnc_bin") and \
             self.__dict__.has_key("app_vnc_options"):
             # create actions and fill them with old settings
-            self.actions["SSH"] = Action(name="SSH", type="command", description="Converted from pre 0.9.9 Nagstamon.",\
+            self.actions["SSH"] = Action(name="SSH", type="command", description="Converted from pre 0.9.9 Nagstamon.",
                                          string=self.app_ssh_bin + " " + self.app_ssh_options + " $ADDRESS$")
 
-            self.actions["RDP"] = Action(name="RDP", type="command", description="Converted from pre 0.9.9 Nagstamon.",\
+            self.actions["RDP"] = Action(name="RDP", type="command", description="Converted from pre 0.9.9 Nagstamon.",
                                          string=self.app_rdp_bin + " " + self.app_rdp_options + " $ADDRESS$")
 
-            self.actions["VNC"] = Action(name="VNC", type="command", description="Converted from pre 0.9.9 Nagstamon.",\
+            self.actions["VNC"] = Action(name="VNC", type="command", description="Converted from pre 0.9.9 Nagstamon.",
                                          string=self.app_vnc_bin + " " + self.app_vnc_options + " $ADDRESS$")
 
             # delete old settings from config
@@ -581,65 +659,73 @@ class Config(object):
         create some default actions like SSH and so on
         """
         if platform.system() == "Windows":
-            defaultactions = { "RDP": Action(name="RDP", description="Connect via RDP.",\
-                                    type="command", string="C:\windows\system32\mstsc.exe $ADDRESS$"),\
-                               "VNC": Action(name="VNC", description="Connect via VNC.",\
-                                    type="command", string="C:\Program Files\TightVNC\vncviewer.exe $ADDRESS$"),\
-                               "Telnet": Action(name="Telnet", description="Connect via Telnet.",\
-                                    type="command", string="C:\Windows\System32\Telnet.exe root@$ADDRESS$"),\
-                               "SSH": Action(name="SSH", description="Connect via SSH.",\
-                                    type="command", string="C:\Program Files\PuTTY\putty.exe -l root $ADDRESS$")\
+            defaultactions = { "RDP": Action(name="RDP", description="Connect via RDP.",
+                                    type="command", string="C:\windows\system32\mstsc.exe $ADDRESS$"),
+                               "VNC": Action(name="VNC", description="Connect via VNC.",
+                                    type="command", string="C:\Program Files\TightVNC\vncviewer.exe $ADDRESS$"),
+                               "Telnet": Action(name="Telnet", description="Connect via Telnet.",
+                                    type="command", string="C:\Windows\System32\Telnet.exe root@$ADDRESS$"),
+                               "SSH": Action(name="SSH", description="Connect via SSH.",
+                                    type="command", string="C:\Program Files\PuTTY\putty.exe -l root $ADDRESS$")
                                }
         elif platform.system() == "Darwin":
-            defaultactions = { "RDP": Action(name="RDP", description="Connect via RDP.",\
-                                    type="command", string="open rdp://$ADDRESS$"), \
-                               "VNC": Action(name="VNC", description="Connect via VNC.",\
-                                    type="command", string="open vnc://$ADDRESS$"), \
-                               "SSH": Action(name="SSH", description="Connect via SSH.",\
-                                    type="command", string="open ssh://root@$ADDRESS$"),\
-                               "Telnet": Action(name="Telnet", description="Connect via Telnet.",\
-                                    type="command", string="open telnet://root@$ADDRESS$")\
+            defaultactions = { "RDP": Action(name="RDP", description="Connect via RDP.",
+                                    type="command", string="open rdp://$ADDRESS$"),
+                               "VNC": Action(name="VNC", description="Connect via VNC.",
+                                    type="command", string="open vnc://$ADDRESS$"),
+                               "SSH": Action(name="SSH", description="Connect via SSH.",
+                                    type="command", string="open ssh://root@$ADDRESS$"),
+                               "Telnet": Action(name="Telnet", description="Connect via Telnet.",
+                                    type="command", string="open telnet://root@$ADDRESS$")
                                }
         else:
             # the Linux settings
-            defaultactions = { "RDP": Action(name="RDP", description="Connect via RDP.",\
-                                    type="command", string="${LOCALBASE}/bin/rdesktop -g 1024x768 $ADDRESS$"),\
-                               "VNC": Action(name="VNC", description="Connect via VNC.",\
-                                    type="command", string="${LOCALBASE}/bin/vncviewer $ADDRESS$"),\
-                               "SSH": Action(name="SSH", description="Connect via SSH.",\
-                                    type="command", string="${X11BASE}/bin/xterm -e ssh root@$ADDRESS$"),\
-                               "Telnet": Action(name="Telnet", description="Connect via Telnet.",\
-                                    type="command", string="${X11BASE}/bin/xterm -e telnet root@$ADDRESS$"),\
-                               "Update-Linux": Action(name="Update-Linux", description="Run remote update script.",\
-                                    type="command", string="${LOCALBASE}/bin/terminator -x ssh root@$HOST$ update.sh",\
-                                    enabled=False)\
+            defaultactions = { "RDP": Action(name="RDP", description="Connect via RDP.",
+                                    type="command", string="${LOCALBASE}/bin/rdesktop -g 1024x768 $ADDRESS$"),
+                               "VNC": Action(name="VNC", description="Connect via VNC.",
+                                    type="command", string="${LOCALBASE}/bin/vncviewer $ADDRESS$"),
+                               "SSH": Action(name="SSH", description="Connect via SSH.",
+                                    type="command", string="${X11BASE}/bin/xterm -e ssh root@$ADDRESS$"),
+                               "Telnet": Action(name="Telnet", description="Connect via Telnet.",
+                                    type="command", string="${X11BASE}/bin/xterm -e telnet root@$ADDRESS$"),
+                               "Update-Linux": Action(name="Update-Linux", description="Run remote update script.",
+                                    type="command", string="${LOCALBASE}/bin/terminator -x ssh root@$HOST$ update.sh",
+                                    enabled=False)
                                }
         # OS agnostic actions as examples
-        defaultactions["Nagios-1-Click-Acknowledge-Host"] = Action(name="Nagios-1-Click-Acknowledge-Host", type="url",\
-                                                    description="Acknowledges a host with one click.",\
-                                                    filter_target_service=False, enabled=False,\
+        defaultactions["Nagios-1-Click-Acknowledge-Host"] = Action(name="Nagios-1-Click-Acknowledge-Host", type="url",
+                                                    description="Acknowledges a host with one click.",
+                                                    filter_target_service=False, enabled=False,
                                                     string="$MONITOR-CGI$/cmd.cgi?cmd_typ=33&cmd_mod=2&host=$HOST$\
                                                     &com_author=$USERNAME$&com_data=acknowledged&btnSubmit=Commit")
-        defaultactions["Nagios-1-Click-Acknowledge-Service"] = Action(name="Nagios-1-Click-Acknowledge-Service", type="url",\
-                                                    description="Acknowledges a service with one click.",\
-                                                    filter_target_host=False, enabled=False,\
+        defaultactions["Nagios-1-Click-Acknowledge-Service"] = Action(name="Nagios-1-Click-Acknowledge-Service", type="url",
+                                                    description="Acknowledges a service with one click.",
+                                                    filter_target_host=False, enabled=False,
                                                     string="$MONITOR-CGI$/cmd.cgi?cmd_typ=34&cmd_mod=2&host=$HOST$\
                                                     &service=$SERVICE$&com_author=$USERNAME$&com_data=acknowledged&btnSubmit=Commit")
-        defaultactions["Opsview-Graph-Service"] = Action(name="Opsview-Graph-Service", type="browser",\
-                                                    description="Show graph in browser.", filter_target_host=False,\
+        defaultactions["Opsview-Graph-Service"] = Action(name="Opsview-Graph-Service", type="browser",
+                        description="Show graph in browser.", filter_target_host=False,
                                                     string="$MONITOR$/graph?service=$SERVICE$&host=$HOST$", enabled=False)
-        defaultactions["Opsview-History-Host"] = Action(name="Opsview-Host-Service", type="browser",\
-                                                    description="Show host in browser.", filter_target_host=True,\
+        defaultactions["Opsview-History-Host"] = Action(name="Opsview-Host-Service", type="browser",
+                                                    description="Show host in browser.", filter_target_host=True,
                                                     string="$MONITOR$/event?host=$HOST$", enabled=False)
-        defaultactions["Opsview-History-Service"] = Action(name="Opsview-History-Service", type="browser",\
-                                                    description="Show history in browser.", filter_target_host=True,\
+        defaultactions["Opsview-History-Service"] = Action(name="Opsview-History-Service", type="browser",
+                                                    description="Show history in browser.", filter_target_host=True,
                                                     string="$MONITOR$/event?host=$HOST$&service=$SERVICE$", enabled=False)
-        defaultactions["Ninja-Configure-Host"] = Action(name="Ninja-Configure-Host", type="browser",\
-                                                    description="Configure host in browser.",\
-                                                    string="$MONITOR$/index.php/configuration/configure/host/$HOST$", enabled=False)
-        defaultactions["Ninja-Configure-Service"] = Action(name="Ninja-Configure-Service", type="browser", filter_target_host=False,\
-                                                    description="Configure service in browser.",\
-                                                    string="$MONITOR$/index.php/configuration/configure/service/$HOST$?service=$SERVICE$", enabled=False)
+        defaultactions["Check_MK-1-Click-Acknowledge-Host"] = Action(name="Check_MK-1-Click-Acknowledge-Host", type="url",
+                                                    description="Acknowledges a host with one click.",
+                                                    filter_target_service=False, enabled=False,
+                                                    string="$MONITOR$/view.py?_transid=$TRANSID$&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=hoststatus&host=$HOST$&_ack_comment=$COMMENT-ACK$&_acknowledge=Acknowledge")
+        defaultactions["Check_MK-1-Click-Acknowledge-Service"] = Action(name="Check_MK-1-Click-Acknowledge-Service", type="url",
+                                                    description="Acknowledges a host with one click.",
+                                                    filter_target_host=False, enabled=False,
+                                                    string="$MONITOR$/view.py?_transid=$TRANSID$&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=service&host=$HOST$&_ack_comment=$COMMENT-ACK$&_acknowledge=Acknowledge&service=$SERVICE$")
+        defaultactions["Check_MK Edit host in WATO"] = Action(name="Check_MK Edit host in WATO", enabled=True,
+                                                     monitor_type="Check_MK Multisite",
+                                                     description="Edit host in WATO.",
+                                                     string="$MONITOR$/index.py?start_url=%2Fmonitor%2Fcheck_mk%2Fwato.py%3Fmode%3Dedithost%26host%3D$HOST$")
+        defaultactions["Email"] = Action(name="Email", enabled=False, description="Send email to someone.", type="browser",
+                                                    string="mailto:servicedesk@my.org?subject=Monitor alert: $HOST$ - $SERVICE$ - $STATUS-INFO$&body=Please help!.%0d%0aBest regards from Nagstamon")
 
         return defaultactions
 
@@ -698,9 +784,16 @@ class Server(object):
         self.proxy_address = ""
         self.proxy_username = ""
         self.proxy_password = ""
+
+        # special FX
+        # Centreon autologin
         self.use_autologin = False
         self.autologin_key = ""
 
+        # Icinga "host_display_name" instead of "host"
+        self.use_display_name_host = False
+        self.use_display_name_service = False
+
 
 class Action(object):
     """
@@ -710,6 +803,8 @@ class Action(object):
     def __init__(self, **kwds):
         # to be or not to be enabled...
         self.enabled = True
+        # monitor type
+        self.monitor_type = ""
         # one of those: browser, url or command
         self.type = "browser"
         # thy name is...
@@ -735,12 +830,15 @@ class Action(object):
         self.re_status_information_enabled = False
         self.re_status_information_pattern = ""
         self.re_status_information_reverse = False
-        self.re_criticality_enabled = False
-        self.re_criticality_pattern = ""
-        self.re_criticality_reverse = False
         # close powin or not, depends on personal preference
         self.close_popwin = True
         self.leave_popwin_open = False
 
+        # special FX
+        # Centreon criticality and autologin
+        self.re_criticality_enabled = False
+        self.re_criticality_pattern = ""
+        self.re_criticality_reverse = False
+
         # add and/or all keywords to object
         for k in kwds: self.__dict__[k] = kwds[k]
